import struct
import binascii
import logging
from gzip import GzipFile
from time import time
from collections import defaultdict
from itertools import cycle, count

from io import BytesIO

import gevent
import gevent.socket as socket
from random import shuffle

from steam.steamid import SteamID
from steam.enums import EResult, EUniverse
from steam.enums.emsg import EMsg
from steam.core import crypto
from steam.core.connection import TCPConnection
from steam.core.msg import Msg, MsgProto
from eventemitter import EventEmitter
from steam.utils import ip4_from_int
from steam.utils.proto import is_proto, clear_proto_bit


class CMClient(EventEmitter):
    """
    CMClient provides a secure message channel to Steam CM servers
    Can be used as mixing or on it's own.

    Incoming messages are parsed and emitted as events using
    their :class:`steam.enums.emsg.EMsg` as event identifier
    """

    EVENT_CONNECTED = 'connected'
    """Connection establed to CM server
    """
    EVENT_DISCONNECTED = 'disconnected'
    """Connection closed
    """
    EVENT_RECONNECT = 'reconnect'
    """Delayed connect

    :param delay: delay in seconds
    :type delay: int
    """
    EVENT_CHANNEL_SECURED = 'channel_secured'
    """After successful completion of encryption handshake
    """
    EVENT_ERROR = 'error'
    """When login is denied

    :param eresult: reason
    :type eresult: :class:`.EResult`
    """
    EVENT_EMSG = 0
    """All incoming messages are emitted with their :class:`.EMsg` number.
    """

    PROTOCOL_TCP = 0                        #: TCP protocol enum
    PROTOCOL_UDP = 1                        #: UDP protocol enum
    verbose_debug = False                   #: print message connects in debug

    auto_discovery = True                   #: enables automatic CM discovery
    cm_servers = None                       #: a instance of :class:`.CMServerList`
    current_server_addr = None              #: (ip, port) tuple
    _seen_logon = False
    _connecting = False
    connected = False                       #: :class:`True` if connected to CM
    channel_secured = False                 #: :class:`True` once secure channel handshake is complete

    channel_key = None                      #: channel encryption key
    channel_hmac = None                     #: HMAC secret

    steam_id = SteamID()                    #: :class:`.SteamID` of the current user
    session_id = None                       #: session id when logged in
    cell_id = 0                             #: cell id provided by CM

    _recv_loop = None
    _heartbeat_loop = None
    _LOG = logging.getLogger("CMClient")

    def __init__(self, protocol=PROTOCOL_TCP):
        self.cm_servers = CMServerList()

        if protocol == CMClient.PROTOCOL_TCP:
            self.connection = TCPConnection()
        else:
            raise ValueError("Only TCP is supported")

        self.on(EMsg.ChannelEncryptRequest, self.__handle_encrypt_request),
        self.on(EMsg.Multi, self.__handle_multi),
        self.on(EMsg.ClientLogOnResponse, self._handle_logon),
        self.on(EMsg.ClientCMList, self._handle_cm_list),

    def emit(self, event, *args):
        if event is not None:
            self._LOG.debug("Emit event: %s" % repr(event))
        super(CMClient, self).emit(event, *args)

    def connect(self, retry=0, delay=0):
        """Initiate connection to CM. Blocks until connected unless ``retry`` is specified.

        :param retry: number of retries before returning. Unlimited when set to ``None``
        :type retry: :class:`int`
        :param delay: delay in seconds before connection attempt
        :type delay: :class:`int`
        :return: successful connection
        :rtype: :class:`bool`
        """
        if self.connected:
            self._LOG.debug("Connect called, but we are connected?")
            return
        if self._connecting:
            self._LOG.debug("Connect called, but we are already connecting.")
            return
        self._connecting = True

        if delay:
            self._LOG.debug("Delayed connect: %d seconds" % delay)
            self.emit(self.EVENT_RECONNECT, delay)
            self.sleep(delay)

        self._LOG.debug("Connect initiated.")

        i = count(0)

        while len(self.cm_servers) == 0:
            if not self.auto_discovery or (retry and next(i) >= retry):
                if not self.auto_discovery:
                    self._LOG.error("CM server list is empty. Auto discovery is off.")
                self._connecting = False
                return False

            if not self.cm_servers.bootstrap_from_webapi():
                self.cm_servers.bootstrap_from_dns()

        for i, server_addr in enumerate(cycle(self.cm_servers), start=next(i)-1):
            if retry and i >= retry:
                self._connecting = False
                return False

            start = time()

            if self.connection.connect(server_addr):
                break
            self._LOG.debug("Failed to connect. Retrying...")

            diff = time() - start

            if diff < 5:
                self.sleep(5 - diff)

        self.current_server_addr = server_addr
        self.connected = True
        self.emit(self.EVENT_CONNECTED)
        self._recv_loop = gevent.spawn(self._recv_messages)
        self._connecting = False
        return True

    def disconnect(self):
        """Close connection"""

        if not self.connected:
            return
        self.connected = False

        self.connection.disconnect()

        if self._heartbeat_loop:
            self._heartbeat_loop.kill()
        self._recv_loop.kill()

        self._reset_attributes()

        self.emit(self.EVENT_DISCONNECTED)

    def _reset_attributes(self):
        for name in ['connected',
                     'channel_secured',
                     'channel_key',
                     'channel_hmac',
                     'steam_id',
                     'session_id',
                     '_seen_logon',
                     '_recv_loop',
                     '_heartbeat_loop',
                     ]:
            self.__dict__.pop(name, None)

    def send(self, message):
        """
        Send a message

        :param message: a message instance
        :type message: :class:`steam.core.msg.Msg`, :class:`steam.core.msg.MsgProto`
        """
        if not isinstance(message, (Msg, MsgProto)):
            raise ValueError("Expected Msg or MsgProto, got %s" % message)

        if self.steam_id:
            message.steamID = self.steam_id
        if self.session_id:
            message.sessionID = self.session_id

        if self.verbose_debug:
            self._LOG.debug("Outgoing: %s\n%s" % (repr(message), str(message)))
        else:
            self._LOG.debug("Outgoing: %s", repr(message))

        data = message.serialize()

        if self.channel_key:
            if self.channel_hmac:
                data = crypto.symmetric_encrypt_HMAC(data, self.channel_key, self.channel_hmac)
            else:
                data = crypto.symmetric_encrypt(data, self.channel_key)

        self.connection.put_message(data)

    def _recv_messages(self):
        for message in self.connection:
            if not self.connected:
                break

            if self.channel_key:
                if self.channel_hmac:
                    try:
                        message = crypto.symmetric_decrypt_HMAC(message, self.channel_key, self.channel_hmac)
                    except RuntimeError as e:
                        self._LOG.exception(e)
                        break
                else:
                    message = crypto.symmetric_decrypt(message, self.channel_key)

            gevent.spawn(self._parse_message, message)
            self.idle()

        if not self._seen_logon and self.channel_secured:
            if self.wait_event('disconnected', timeout=5) is not None:
                return

        gevent.spawn(self.disconnect)

    def _parse_message(self, message):
        emsg_id, = struct.unpack_from("<I", message)
        emsg = EMsg(clear_proto_bit(emsg_id))

        if not self.connected and emsg != EMsg.ClientLogOnResponse:
            self._LOG.debug("Dropped unexpected message: %s (is_proto: %s)",
                            repr(emsg),
                            is_proto(emsg_id),
                            )
            return

        if emsg in (EMsg.ChannelEncryptRequest,
                    EMsg.ChannelEncryptResponse,
                    EMsg.ChannelEncryptResult,
                    ):

            msg = Msg(emsg, message, parse=False)
        else:
            try:
                if is_proto(emsg_id):
                    msg = MsgProto(emsg, message, parse=False)
                else:
                    msg = Msg(emsg, message, extended=True, parse=False)
            except Exception as e:
                self._LOG.fatal("Failed to deserialize message: %s (is_proto: %s)",
                                repr(emsg),
                                is_proto(emsg_id)
                                )
                self._LOG.exception(e)
                return

        if self.count_listeners(emsg) or self.verbose_debug:
            msg.parse()

        if self.verbose_debug:
            self._LOG.debug("Incoming: %s\n%s" % (repr(msg), str(msg)))
        else:
            self._LOG.debug("Incoming: %s", repr(msg))

        self.emit(emsg, msg)
        return emsg, msg

    def __handle_encrypt_request(self, req):
        self._LOG.debug("Securing channel")

        try:
            if req.body.protocolVersion != 1:
                raise RuntimeError("Unsupported protocol version")
            if req.body.universe != EUniverse.Public:
                raise RuntimeError("Unsupported universe")
        except RuntimeError as e:
            self._LOG.exception(e)
            gevent.spawn(self.disconnect)
            return

        resp = Msg(EMsg.ChannelEncryptResponse)

        challenge = req.body.challenge
        key, resp.body.key = crypto.generate_session_key(challenge)
        resp.body.crc = binascii.crc32(resp.body.key) & 0xffffffff

        self.send(resp)

        result = self.wait_event(EMsg.ChannelEncryptResult, timeout=5)

        if result is None:
            self.cm_servers.mark_bad(self.current_server_addr)
            gevent.spawn(self.disconnect)
            return

        eresult = result[0].body.eresult

        if eresult != EResult.OK:
            self._LOG.error("Failed to secure channel: %s" % eresult)
            gevent.spawn(self.disconnect)
            return

        self.channel_key = key

        if challenge:
            self._LOG.debug("Channel secured")
            self.channel_hmac = key[:16]
        else:
            self._LOG.debug("Channel secured (legacy mode)")

        self.channel_secured = True
        self.emit(self.EVENT_CHANNEL_SECURED)

    def __handle_multi(self, msg):
        self._LOG.debug("Multi: Unpacking")

        if msg.body.size_unzipped:
            self._LOG.debug("Multi: Decompressing payload (%d -> %s)" % (
                len(msg.body.message_body),
                msg.body.size_unzipped,
                ))

            with GzipFile(fileobj=BytesIO(msg.body.message_body)) as f:
                data = f.read()

            if len(data) != msg.body.size_unzipped:
                self._LOG.fatal("Unzipped size mismatch")
                gevent.spawn(self.disconnect)
                return
        else:
            data = msg.body.message_body

        while len(data) > 0:
            size, = struct.unpack_from("<I", data)
            self._parse_message(data[4:4+size])
            data = data[4+size:]

    def __heartbeat(self, interval):
        message = MsgProto(EMsg.ClientHeartBeat)

        while True:
            self.sleep(interval)
            self.send(message)

    def _handle_logon(self, msg):
        result = msg.body.eresult

        if result in (EResult.TryAnotherCM,
                      EResult.ServiceUnavailable
                      ):
            self.cm_servers.mark_bad(self.current_server_addr)
            self.disconnect()
        elif result == EResult.OK:
            self._seen_logon = True

            self._LOG.debug("Logon completed")

            self.steam_id = SteamID(msg.header.steamid)
            self.session_id = msg.header.client_sessionid
            self.cell_id = msg.body.cell_id

            if self._heartbeat_loop:
                self._heartbeat_loop.kill()

            self._LOG.debug("Heartbeat started.")

            interval = msg.body.out_of_game_heartbeat_seconds
            self._heartbeat_loop = gevent.spawn(self.__heartbeat, interval)
        else:
            self.emit(self.EVENT_ERROR, EResult(result))
            self.disconnect()

    def _handle_cm_list(self, msg):
        self._LOG.debug("Updating CM list")

        new_servers = zip(map(ip4_from_int, msg.body.cm_addresses), msg.body.cm_ports)
        self.cm_servers.clear()
        self.cm_servers.merge_list(new_servers)
        self.cm_servers.cell_id = self.cell_id

    def sleep(self, seconds):
        """Yeild and sleep N seconds. Allows other greenlets to run"""
        gevent.sleep(seconds)

    def idle(self):
        """Yeild in the current greenlet and let other greenlets run"""
        gevent.idle()


class CMServerList(object):
    """
    Managing object for CM servers

    Comes with built in list of CM server to bootstrap a connection

    To get a server address from the list simply iterate over it

    .. code:: python

        servers = CMServerList()
        for server_addr in servers:
            pass

    The good servers are returned first, then bad ones. After failing to connect
    call :meth:`mark_bad` with the server addr. When connection succeeds break
    out of the loop.
    """

    Good = 1
    Bad = 2
    last_updated = 0       #: timestamp of when the list was last updated
    cell_id = 0            #: cell id of the server list
    bad_timestamp = 300    #: how long bad mark lasts in seconds

    def __init__(self):
        self._LOG = logging.getLogger("CMServerList")
        self.list = defaultdict(dict)

    def __len__(self):
        return len(self.list)

    def __repr__(self):
        return "<%s: %d servers>" % (self.__class__.__name__, len(self))

    def clear(self):
        """Clears the server list"""
        if len(self.list):
            self._LOG.debug("List cleared.")
        self.list.clear()

    def bootstrap_from_dns(self):
        """
        Fetches CM server list from WebAPI and replaces the current one
        """
        self._LOG.debug("Attempting bootstrap via DNS")

        try:
            answer = socket.getaddrinfo("cm0.steampowered.com",
                                        27017,
                                        socket.AF_INET,
                                        proto=socket.IPPROTO_TCP)
        except Exception as exp:
            self._LOG.error("DNS boostrap failed: %s" % str(exp))
            return False

        servers = list(map(lambda addr: addr[4], answer))

        if servers:
            self.clear()
            self.merge_list(servers)
            return True
        else:
            self._LOG.error("DNS boostrap: cm0.steampowered.com resolved no A records")
            return False

    def bootstrap_from_webapi(self, cell_id=0):
        """
        Fetches CM server list from WebAPI and replaces the current one

        :param cellid: cell id (0 = global)
        :type cellid: :class:`int`
        :return: booststrap success
        :rtype: :class:`bool`
        """
        self._LOG.debug("Attempting bootstrap via WebAPI")

        from steam import webapi
        try:
            resp = webapi.get('ISteamDirectory', 'GetCMList', 1, params={'cellid': cell_id,
                                                                         'http_timeout': 3})
        except Exception as exp:
            self._LOG.error("WebAPI boostrap failed: %s" % str(exp))
            return False

        result = EResult(resp['response']['result'])

        if result != EResult.OK:
            self._LOG.error("GetCMList failed with %s" % repr(result))
            return False

        serverlist = resp['response']['serverlist']
        self._LOG.debug("Received %d servers from WebAPI" % len(serverlist))

        def str_to_tuple(serveraddr):
            ip, port = serveraddr.split(':')
            return str(ip), int(port)

        self.clear()
        self.cell_id = cell_id
        self.merge_list(map(str_to_tuple, serverlist))

        return True

    def __iter__(self):
        def cm_server_iter():
            if not self.list:
                self._LOG.error("Server list is empty.")
                return

            good_servers = list(filter(lambda x: x[1]['quality'] == CMServerList.Good,
                                       self.list.items()
                                       ))

            if len(good_servers) == 0:
                self._LOG.debug("No good servers left. Reseting...")
                self.reset_all()
                return

            shuffle(good_servers)

            for server_addr, meta in good_servers:
                yield server_addr

        return cm_server_iter()

    def reset_all(self):
        """Reset status for all servers in the list"""

        self._LOG.debug("Marking all CMs as Good.")

        for key in self.list:
            self.mark_good(key)

    def mark_good(self, server_addr):
        """Mark server address as good

        :param server_addr: (ip, port) tuple
        :type server_addr: :class:`tuple`
        """
        self.list[server_addr].update({'quality': CMServerList.Good, 'timestamp': time()})

    def mark_bad(self, server_addr):
        """Mark server address as bad, when unable to connect for example

        :param server_addr: (ip, port) tuple
        :type server_addr: :class:`tuple`
        """
        self._LOG.debug("Marking %s as Bad." % repr(server_addr))
        self.list[server_addr].update({'quality': CMServerList.Bad, 'timestamp': time()})

    def merge_list(self, new_list):
        """Add new CM servers to the list

        :param new_list: a list of ``(ip, port)`` tuples
        :type new_list: :class:`list`
        """
        total = len(self.list)

        for ip, port in new_list:
            if (ip, port) not in self.list:
                self.mark_good((ip, port))

        if len(self.list) > total:
            self._LOG.debug("Added %d new CM addresses." % (len(self.list) - total))

        self.last_updated = int(time())



"""
All function in this module take and return :class:`bytes`
"""
import sys
from os import urandom as random_bytes
from struct import pack
from base64 import b64decode

from Cryptodome.Hash import MD5, SHA1, HMAC
from Cryptodome.PublicKey.RSA import import_key as rsa_import_key, construct as rsa_construct
from Cryptodome.Cipher import PKCS1_OAEP, PKCS1_v1_5
from Cryptodome.Cipher import AES as AES


class UniverseKey(object):
    """Public keys for Universes"""

    Public = rsa_import_key(b64decode("""
MIGdMA0GCSqGSIb3DQEBAQUAA4GLADCBhwKBgQDf7BrWLBBmLBc1OhSwfFkRf53T
2Ct64+AVzRkeRuh7h3SiGEYxqQMUeYKO6UWiSRKpI2hzic9pobFhRr3Bvr/WARvY
gdTckPv+T1JzZsuVcNfFjrocejN1oWI0Rrtgt4Bo+hOneoo3S57G9F1fOpn5nsQ6
6WOiu4gZKODnFMBCiQIBEQ==
"""))

BS = 16
pad = lambda s: s + (BS - len(s) % BS) * pack('B', BS - len(s) % BS)

if sys.version_info < (3,):
    unpad = lambda s: s[0:-ord(s[-1])]
else:
    unpad = lambda s: s[0:-s[-1]]


def generate_session_key(hmac_secret=b''):
    """
    :param hmac_secret: optional HMAC
    :type hmac_secret: :class:`bytes`
    :return: (session_key, encrypted_session_key) tuple
    :rtype: :class:`tuple`
    """
    session_key = random_bytes(32)
    encrypted_session_key = PKCS1_OAEP.new(UniverseKey.Public, SHA1)\
                                      .encrypt(session_key + hmac_secret)

    return (session_key, encrypted_session_key)

def symmetric_encrypt(message, key):
    iv = random_bytes(BS)
    return symmetric_encrypt_with_iv(message, key, iv)

def symmetric_encrypt_ecb(message, key):
    return AES.new(key, AES.MODE_ECB).encrypt(pad(message))

def symmetric_encrypt_HMAC(message, key, hmac_secret):
    prefix = random_bytes(3)
    hmac = hmac_sha1(hmac_secret, prefix + message)
    iv = hmac[:13] + prefix
    return symmetric_encrypt_with_iv(message, key, iv)

def symmetric_encrypt_iv(iv, key):
    return AES.new(key, AES.MODE_ECB).encrypt(iv)

def symmetric_encrypt_with_iv(message, key, iv):
    encrypted_iv = symmetric_encrypt_iv(iv, key)
    cyphertext = AES.new(key, AES.MODE_CBC, iv).encrypt(pad(message))
    return encrypted_iv + cyphertext

def symmetric_decrypt(cyphertext, key):
    iv = symmetric_decrypt_iv(cyphertext, key)
    return symmetric_decrypt_with_iv(cyphertext, key, iv)

def symmetric_decrypt_ecb(cyphertext, key):
    return unpad(AES.new(key, AES.MODE_ECB).decrypt(cyphertext))

def symmetric_decrypt_HMAC(cyphertext, key, hmac_secret):
    """:raises: :class:`RuntimeError` when HMAC verification fails"""
    iv = symmetric_decrypt_iv(cyphertext, key)
    message = symmetric_decrypt_with_iv(cyphertext, key, iv)

    hmac = hmac_sha1(hmac_secret, iv[-3:] + message)

    if iv[:13] != hmac[:13]:
        raise RuntimeError("Unable to decrypt message. HMAC does not match.")

    return message

def symmetric_decrypt_iv(cyphertext, key):
    return AES.new(key, AES.MODE_ECB).decrypt(cyphertext[:BS])

def symmetric_decrypt_with_iv(cyphertext, key, iv):
    return unpad(AES.new(key, AES.MODE_CBC, iv).decrypt(cyphertext[BS:]))

def hmac_sha1(secret, data):
    return HMAC.new(secret, data, SHA1).digest()

def sha1_hash(data):
    return SHA1.new(data).digest()

def md5_hash(data):
    return MD5.new(data).digest()

def rsa_publickey(mod, exp):
    return rsa_construct((mod, exp))

def pkcs1v15_encrypt(key, message):
    return PKCS1_v1_5.new(key).encrypt(message)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: steammessages_site_license.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import service as _service
from google.protobuf import service_reflection
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import steam.protobufs.steammessages_base_pb2 as steammessages__base__pb2
import steam.protobufs.steammessages_unified_base_pb2 as steammessages__unified__base__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='steammessages_site_license.proto',
  package='',
  syntax='proto2',
  serialized_options=b'\220\001\001',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n steammessages_site_license.proto\x1a\x18steammessages_base.proto\x1a steammessages_unified_base.proto\"\x8d\x01\n)CSiteManagerClient_IncomingClient_Request\x12\x17\n\x0fsite_instanceid\x18\x01 \x01(\x06\x12\x16\n\x0e\x63lient_steamid\x18\x02 \x01(\x06\x12\x17\n\x0f\x63lient_local_ip\x18\x03 \x01(\x07\x12\x16\n\x0e\x63onnection_key\x18\x04 \x01(\x0c\",\n*CSiteManagerClient_IncomingClient_Response\"N\n,CSiteLicense_ClientSeatCheckout_Notification\x12\r\n\x05\x61ppid\x18\x01 \x01(\r\x12\x0f\n\x07\x65result\x18\x02 \x01(\r\"\xe3\x02\n/CSiteManagerClient_TrackedPayments_Notification\x12\x0f\n\x07site_id\x18\x01 \x01(\x06\x12J\n\x08payments\x18\x02 \x03(\x0b\x32\x38.CSiteManagerClient_TrackedPayments_Notification.Payment\x1a\xd2\x01\n\x07Payment\x12\x0f\n\x07transid\x18\x01 \x01(\x04\x12\x0f\n\x07steamid\x18\x02 \x01(\x04\x12\x0e\n\x06\x61mount\x18\x03 \x01(\x03\x12\x11\n\tecurrency\x18\x04 \x01(\r\x12\x14\n\x0ctime_created\x18\x05 \x01(\x05\x12\x17\n\x0fpurchase_status\x18\x06 \x01(\x05\x12\x14\n\x0cmachine_name\x18\x07 \x01(\t\x12\x14\n\x0cpersona_name\x18\x08 \x01(\t\x12\x13\n\x0bprofile_url\x18\t \x01(\t\x12\x12\n\navatar_url\x18\n \x01(\t\"r\n(CSiteLicense_InitiateAssociation_Request\x12\x14\n\x0csite_steamid\x18\x01 \x01(\x06\x12\x17\n\x0fsite_instanceid\x18\x02 \x01(\x06\x12\x17\n\x0f\x63lient_local_ip\x18\x03 \x01(\x07\"C\n)CSiteLicense_InitiateAssociation_Response\x12\x16\n\x0e\x63onnection_key\x18\x01 \x01(\x0c\":\n$CSiteLicense_LCSAuthenticate_Request\x12\x12\n\ninstanceid\x18\x01 \x01(\x06\"z\n%CSiteLicense_LCSAuthenticate_Response\x12\x0f\n\x07site_id\x18\x01 \x01(\x04\x12\x11\n\tsite_name\x18\x02 \x01(\t\x12\x13\n\x0bnew_session\x18\x03 \x01(\x08\x12\x18\n\x10no_site_licenses\x18\x04 \x01(\x08\"t\n%CSiteLicense_LCSAssociateUser_Request\x12\x0f\n\x07steamid\x18\x01 \x01(\x06\x12\x10\n\x08local_ip\x18\x02 \x01(\x07\x12\x12\n\ninstanceid\x18\x03 \x01(\x06\x12\x14\n\x0cmachine_name\x18\x04 \x01(\t\"(\n&CSiteLicense_LCSAssociateUser_Response\"]\n\'CSiteLicense_ClientSeatCheckout_Request\x12\x0f\n\x07steamid\x18\x01 \x01(\x06\x12\x12\n\ninstanceid\x18\x02 \x01(\x06\x12\r\n\x05\x61ppid\x18\x03 \x01(\r\"*\n(CSiteLicense_ClientSeatCheckout_Response\"b\n,CSiteLicense_ClientGetAvailableSeats_Request\x12\x0f\n\x07steamid\x18\x01 \x01(\x06\x12\x12\n\ninstanceid\x18\x02 \x01(\x06\x12\r\n\x05\x61ppid\x18\x03 \x01(\r\"H\n-CSiteLicense_ClientGetAvailableSeats_Response\x12\x17\n\x0f\x61vailable_seats\x18\x01 \x01(\r2\xa3\x04\n\x11SiteManagerClient\x12\xaf\x01\n\x0eIncomingClient\x12*.CSiteManagerClient_IncomingClient_Request\x1a+.CSiteManagerClient_IncomingClient_Response\"D\x82\xb5\x18@Rack informing site manager of new inbound client PSK connection\x12\x93\x01\n\x1e\x43lientSeatCheckoutNotification\x12-.CSiteLicense_ClientSeatCheckout_Notification\x1a\x0b.NoResponse\"5\x82\xb5\x18\x31Notify client directly about seat checkout result\x12\x9a\x01\n\x1bTrackedPaymentsNotification\x12\x30.CSiteManagerClient_TrackedPayments_Notification\x1a\x0b.NoResponse\"<\x82\xb5\x18\x38Rack informing site manager of existing tracked payments\x1a)\x82\xb5\x18!Local site manager client service\xc0\xb5\x18\x02\x32\xd5\x06\n\x0bSiteLicense\x12\xa4\x01\n\x13InitiateAssociation\x12).CSiteLicense_InitiateAssociation_Request\x1a*.CSiteLicense_InitiateAssociation_Response\"6\x82\xb5\x18\x32\x43lient is requesting connetion info for local site\x12\x86\x01\n\x0fLCSAuthenticate\x12%.CSiteLicense_LCSAuthenticate_Request\x1a&.CSiteLicense_LCSAuthenticate_Response\"$\x82\xb5\x18 Local cafe server authentication\x12\x9d\x01\n\x10LCSAssociateUser\x12&.CSiteLicense_LCSAssociateUser_Request\x1a\'.CSiteLicense_LCSAssociateUser_Response\"8\x82\xb5\x18\x34Local cafe server wants to grant user access to site\x12\x9e\x01\n\x12\x43lientSeatCheckout\x12(.CSiteLicense_ClientSeatCheckout_Request\x1a).CSiteLicense_ClientSeatCheckout_Response\"3\x82\xb5\x18/Client is requesting use of a site license seat\x12\xba\x01\n\x17\x43lientGetAvailableSeats\x12-.CSiteLicense_ClientGetAvailableSeats_Request\x1a..CSiteLicense_ClientGetAvailableSeats_Response\"@\x82\xb5\x18<Client is requesting the number of currently available seats\x1a\x18\x82\xb5\x18\x14Site License serviceB\x03\x90\x01\x01'
  ,
  dependencies=[steammessages__base__pb2.DESCRIPTOR,steammessages__unified__base__pb2.DESCRIPTOR,])




_CSITEMANAGERCLIENT_INCOMINGCLIENT_REQUEST = _descriptor.Descriptor(
  name='CSiteManagerClient_IncomingClient_Request',
  full_name='CSiteManagerClient_IncomingClient_Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='site_instanceid', full_name='CSiteManagerClient_IncomingClient_Request.site_instanceid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='client_steamid', full_name='CSiteManagerClient_IncomingClient_Request.client_steamid', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='client_local_ip', full_name='CSiteManagerClient_IncomingClient_Request.client_local_ip', index=2,
      number=3, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='connection_key', full_name='CSiteManagerClient_IncomingClient_Request.connection_key', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=97,
  serialized_end=238,
)


_CSITEMANAGERCLIENT_INCOMINGCLIENT_RESPONSE = _descriptor.Descriptor(
  name='CSiteManagerClient_IncomingClient_Response',
  full_name='CSiteManagerClient_IncomingClient_Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=240,
  serialized_end=284,
)


_CSITELICENSE_CLIENTSEATCHECKOUT_NOTIFICATION = _descriptor.Descriptor(
  name='CSiteLicense_ClientSeatCheckout_Notification',
  full_name='CSiteLicense_ClientSeatCheckout_Notification',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='appid', full_name='CSiteLicense_ClientSeatCheckout_Notification.appid', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='eresult', full_name='CSiteLicense_ClientSeatCheckout_Notification.eresult', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=286,
  serialized_end=364,
)


_CSITEMANAGERCLIENT_TRACKEDPAYMENTS_NOTIFICATION_PAYMENT = _descriptor.Descriptor(
  name='Payment',
  full_name='CSiteManagerClient_TrackedPayments_Notification.Payment',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='transid', full_name='CSiteManagerClient_TrackedPayments_Notification.Payment.transid', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='steamid', full_name='CSiteManagerClient_TrackedPayments_Notification.Payment.steamid', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='amount', full_name='CSiteManagerClient_TrackedPayments_Notification.Payment.amount', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ecurrency', full_name='CSiteManagerClient_TrackedPayments_Notification.Payment.ecurrency', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time_created', full_name='CSiteManagerClient_TrackedPayments_Notification.Payment.time_created', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='purchase_status', full_name='CSiteManagerClient_TrackedPayments_Notification.Payment.purchase_status', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='machine_name', full_name='CSiteManagerClient_TrackedPayments_Notification.Payment.machine_name', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='persona_name', full_name='CSiteManagerClient_TrackedPayments_Notification.Payment.persona_name', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='profile_url', full_name='CSiteManagerClient_TrackedPayments_Notification.Payment.profile_url', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='avatar_url', full_name='CSiteManagerClient_TrackedPayments_Notification.Payment.avatar_url', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=512,
  serialized_end=722,
)

_CSITEMANAGERCLIENT_TRACKEDPAYMENTS_NOTIFICATION = _descriptor.Descriptor(
  name='CSiteManagerClient_TrackedPayments_Notification',
  full_name='CSiteManagerClient_TrackedPayments_Notification',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='site_id', full_name='CSiteManagerClient_TrackedPayments_Notification.site_id', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payments', full_name='CSiteManagerClient_TrackedPayments_Notification.payments', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_CSITEMANAGERCLIENT_TRACKEDPAYMENTS_NOTIFICATION_PAYMENT, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=367,
  serialized_end=722,
)


_CSITELICENSE_INITIATEASSOCIATION_REQUEST = _descriptor.Descriptor(
  name='CSiteLicense_InitiateAssociation_Request',
  full_name='CSiteLicense_InitiateAssociation_Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='site_steamid', full_name='CSiteLicense_InitiateAssociation_Request.site_steamid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='site_instanceid', full_name='CSiteLicense_InitiateAssociation_Request.site_instanceid', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='client_local_ip', full_name='CSiteLicense_InitiateAssociation_Request.client_local_ip', index=2,
      number=3, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=724,
  serialized_end=838,
)


_CSITELICENSE_INITIATEASSOCIATION_RESPONSE = _descriptor.Descriptor(
  name='CSiteLicense_InitiateAssociation_Response',
  full_name='CSiteLicense_InitiateAssociation_Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='connection_key', full_name='CSiteLicense_InitiateAssociation_Response.connection_key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=840,
  serialized_end=907,
)


_CSITELICENSE_LCSAUTHENTICATE_REQUEST = _descriptor.Descriptor(
  name='CSiteLicense_LCSAuthenticate_Request',
  full_name='CSiteLicense_LCSAuthenticate_Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='instanceid', full_name='CSiteLicense_LCSAuthenticate_Request.instanceid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=909,
  serialized_end=967,
)


_CSITELICENSE_LCSAUTHENTICATE_RESPONSE = _descriptor.Descriptor(
  name='CSiteLicense_LCSAuthenticate_Response',
  full_name='CSiteLicense_LCSAuthenticate_Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='site_id', full_name='CSiteLicense_LCSAuthenticate_Response.site_id', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='site_name', full_name='CSiteLicense_LCSAuthenticate_Response.site_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='new_session', full_name='CSiteLicense_LCSAuthenticate_Response.new_session', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='no_site_licenses', full_name='CSiteLicense_LCSAuthenticate_Response.no_site_licenses', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=969,
  serialized_end=1091,
)


_CSITELICENSE_LCSASSOCIATEUSER_REQUEST = _descriptor.Descriptor(
  name='CSiteLicense_LCSAssociateUser_Request',
  full_name='CSiteLicense_LCSAssociateUser_Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='steamid', full_name='CSiteLicense_LCSAssociateUser_Request.steamid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='local_ip', full_name='CSiteLicense_LCSAssociateUser_Request.local_ip', index=1,
      number=2, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='instanceid', full_name='CSiteLicense_LCSAssociateUser_Request.instanceid', index=2,
      number=3, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='machine_name', full_name='CSiteLicense_LCSAssociateUser_Request.machine_name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1093,
  serialized_end=1209,
)


_CSITELICENSE_LCSASSOCIATEUSER_RESPONSE = _descriptor.Descriptor(
  name='CSiteLicense_LCSAssociateUser_Response',
  full_name='CSiteLicense_LCSAssociateUser_Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1211,
  serialized_end=1251,
)


_CSITELICENSE_CLIENTSEATCHECKOUT_REQUEST = _descriptor.Descriptor(
  name='CSiteLicense_ClientSeatCheckout_Request',
  full_name='CSiteLicense_ClientSeatCheckout_Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='steamid', full_name='CSiteLicense_ClientSeatCheckout_Request.steamid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='instanceid', full_name='CSiteLicense_ClientSeatCheckout_Request.instanceid', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='appid', full_name='CSiteLicense_ClientSeatCheckout_Request.appid', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1253,
  serialized_end=1346,
)


_CSITELICENSE_CLIENTSEATCHECKOUT_RESPONSE = _descriptor.Descriptor(
  name='CSiteLicense_ClientSeatCheckout_Response',
  full_name='CSiteLicense_ClientSeatCheckout_Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1348,
  serialized_end=1390,
)


_CSITELICENSE_CLIENTGETAVAILABLESEATS_REQUEST = _descriptor.Descriptor(
  name='CSiteLicense_ClientGetAvailableSeats_Request',
  full_name='CSiteLicense_ClientGetAvailableSeats_Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='steamid', full_name='CSiteLicense_ClientGetAvailableSeats_Request.steamid', index=0,
      number=1, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='instanceid', full_name='CSiteLicense_ClientGetAvailableSeats_Request.instanceid', index=1,
      number=2, type=6, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='appid', full_name='CSiteLicense_ClientGetAvailableSeats_Request.appid', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1392,
  serialized_end=1490,
)


_CSITELICENSE_CLIENTGETAVAILABLESEATS_RESPONSE = _descriptor.Descriptor(
  name='CSiteLicense_ClientGetAvailableSeats_Response',
  full_name='CSiteLicense_ClientGetAvailableSeats_Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='available_seats', full_name='CSiteLicense_ClientGetAvailableSeats_Response.available_seats', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1492,
  serialized_end=1564,
)

_CSITEMANAGERCLIENT_TRACKEDPAYMENTS_NOTIFICATION_PAYMENT.containing_type = _CSITEMANAGERCLIENT_TRACKEDPAYMENTS_NOTIFICATION
_CSITEMANAGERCLIENT_TRACKEDPAYMENTS_NOTIFICATION.fields_by_name['payments'].message_type = _CSITEMANAGERCLIENT_TRACKEDPAYMENTS_NOTIFICATION_PAYMENT
DESCRIPTOR.message_types_by_name['CSiteManagerClient_IncomingClient_Request'] = _CSITEMANAGERCLIENT_INCOMINGCLIENT_REQUEST
DESCRIPTOR.message_types_by_name['CSiteManagerClient_IncomingClient_Response'] = _CSITEMANAGERCLIENT_INCOMINGCLIENT_RESPONSE
DESCRIPTOR.message_types_by_name['CSiteLicense_ClientSeatCheckout_Notification'] = _CSITELICENSE_CLIENTSEATCHECKOUT_NOTIFICATION
DESCRIPTOR.message_types_by_name['CSiteManagerClient_TrackedPayments_Notification'] = _CSITEMANAGERCLIENT_TRACKEDPAYMENTS_NOTIFICATION
DESCRIPTOR.message_types_by_name['CSiteLicense_InitiateAssociation_Request'] = _CSITELICENSE_INITIATEASSOCIATION_REQUEST
DESCRIPTOR.message_types_by_name['CSiteLicense_InitiateAssociation_Response'] = _CSITELICENSE_INITIATEASSOCIATION_RESPONSE
DESCRIPTOR.message_types_by_name['CSiteLicense_LCSAuthenticate_Request'] = _CSITELICENSE_LCSAUTHENTICATE_REQUEST
DESCRIPTOR.message_types_by_name['CSiteLicense_LCSAuthenticate_Response'] = _CSITELICENSE_LCSAUTHENTICATE_RESPONSE
DESCRIPTOR.message_types_by_name['CSiteLicense_LCSAssociateUser_Request'] = _CSITELICENSE_LCSASSOCIATEUSER_REQUEST
DESCRIPTOR.message_types_by_name['CSiteLicense_LCSAssociateUser_Response'] = _CSITELICENSE_LCSASSOCIATEUSER_RESPONSE
DESCRIPTOR.message_types_by_name['CSiteLicense_ClientSeatCheckout_Request'] = _CSITELICENSE_CLIENTSEATCHECKOUT_REQUEST
DESCRIPTOR.message_types_by_name['CSiteLicense_ClientSeatCheckout_Response'] = _CSITELICENSE_CLIENTSEATCHECKOUT_RESPONSE
DESCRIPTOR.message_types_by_name['CSiteLicense_ClientGetAvailableSeats_Request'] = _CSITELICENSE_CLIENTGETAVAILABLESEATS_REQUEST
DESCRIPTOR.message_types_by_name['CSiteLicense_ClientGetAvailableSeats_Response'] = _CSITELICENSE_CLIENTGETAVAILABLESEATS_RESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CSiteManagerClient_IncomingClient_Request = _reflection.GeneratedProtocolMessageType('CSiteManagerClient_IncomingClient_Request', (_message.Message,), {
  'DESCRIPTOR' : _CSITEMANAGERCLIENT_INCOMINGCLIENT_REQUEST,
  '__module__' : 'steammessages_site_license_pb2'
  # @@protoc_insertion_point(class_scope:CSiteManagerClient_IncomingClient_Request)
  })
_sym_db.RegisterMessage(CSiteManagerClient_IncomingClient_Request)

CSiteManagerClient_IncomingClient_Response = _reflection.GeneratedProtocolMessageType('CSiteManagerClient_IncomingClient_Response', (_message.Message,), {
  'DESCRIPTOR' : _CSITEMANAGERCLIENT_INCOMINGCLIENT_RESPONSE,
  '__module__' : 'steammessages_site_license_pb2'
  # @@protoc_insertion_point(class_scope:CSiteManagerClient_IncomingClient_Response)
  })
_sym_db.RegisterMessage(CSiteManagerClient_IncomingClient_Response)

CSiteLicense_ClientSeatCheckout_Notification = _reflection.GeneratedProtocolMessageType('CSiteLicense_ClientSeatCheckout_Notification', (_message.Message,), {
  'DESCRIPTOR' : _CSITELICENSE_CLIENTSEATCHECKOUT_NOTIFICATION,
  '__module__' : 'steammessages_site_license_pb2'
  # @@protoc_insertion_point(class_scope:CSiteLicense_ClientSeatCheckout_Notification)
  })
_sym_db.RegisterMessage(CSiteLicense_ClientSeatCheckout_Notification)

CSiteManagerClient_TrackedPayments_Notification = _reflection.GeneratedProtocolMessageType('CSiteManagerClient_TrackedPayments_Notification', (_message.Message,), {

  'Payment' : _reflection.GeneratedProtocolMessageType('Payment', (_message.Message,), {
    'DESCRIPTOR' : _CSITEMANAGERCLIENT_TRACKEDPAYMENTS_NOTIFICATION_PAYMENT,
    '__module__' : 'steammessages_site_license_pb2'
    # @@protoc_insertion_point(class_scope:CSiteManagerClient_TrackedPayments_Notification.Payment)
    })
  ,
  'DESCRIPTOR' : _CSITEMANAGERCLIENT_TRACKEDPAYMENTS_NOTIFICATION,
  '__module__' : 'steammessages_site_license_pb2'
  # @@protoc_insertion_point(class_scope:CSiteManagerClient_TrackedPayments_Notification)
  })
_sym_db.RegisterMessage(CSiteManagerClient_TrackedPayments_Notification)
_sym_db.RegisterMessage(CSiteManagerClient_TrackedPayments_Notification.Payment)

CSiteLicense_InitiateAssociation_Request = _reflection.GeneratedProtocolMessageType('CSiteLicense_InitiateAssociation_Request', (_message.Message,), {
  'DESCRIPTOR' : _CSITELICENSE_INITIATEASSOCIATION_REQUEST,
  '__module__' : 'steammessages_site_license_pb2'
  # @@protoc_insertion_point(class_scope:CSiteLicense_InitiateAssociation_Request)
  })
_sym_db.RegisterMessage(CSiteLicense_InitiateAssociation_Request)

CSiteLicense_InitiateAssociation_Response = _reflection.GeneratedProtocolMessageType('CSiteLicense_InitiateAssociation_Response', (_message.Message,), {
  'DESCRIPTOR' : _CSITELICENSE_INITIATEASSOCIATION_RESPONSE,
  '__module__' : 'steammessages_site_license_pb2'
  # @@protoc_insertion_point(class_scope:CSiteLicense_InitiateAssociation_Response)
  })
_sym_db.RegisterMessage(CSiteLicense_InitiateAssociation_Response)

CSiteLicense_LCSAuthenticate_Request = _reflection.GeneratedProtocolMessageType('CSiteLicense_LCSAuthenticate_Request', (_message.Message,), {
  'DESCRIPTOR' : _CSITELICENSE_LCSAUTHENTICATE_REQUEST,
  '__module__' : 'steammessages_site_license_pb2'
  # @@protoc_insertion_point(class_scope:CSiteLicense_LCSAuthenticate_Request)
  })
_sym_db.RegisterMessage(CSiteLicense_LCSAuthenticate_Request)

CSiteLicense_LCSAuthenticate_Response = _reflection.GeneratedProtocolMessageType('CSiteLicense_LCSAuthenticate_Response', (_message.Message,), {
  'DESCRIPTOR' : _CSITELICENSE_LCSAUTHENTICATE_RESPONSE,
  '__module__' : 'steammessages_site_license_pb2'
  # @@protoc_insertion_point(class_scope:CSiteLicense_LCSAuthenticate_Response)
  })
_sym_db.RegisterMessage(CSiteLicense_LCSAuthenticate_Response)

CSiteLicense_LCSAssociateUser_Request = _reflection.GeneratedProtocolMessageType('CSiteLicense_LCSAssociateUser_Request', (_message.Message,), {
  'DESCRIPTOR' : _CSITELICENSE_LCSASSOCIATEUSER_REQUEST,
  '__module__' : 'steammessages_site_license_pb2'
  # @@protoc_insertion_point(class_scope:CSiteLicense_LCSAssociateUser_Request)
  })
_sym_db.RegisterMessage(CSiteLicense_LCSAssociateUser_Request)

CSiteLicense_LCSAssociateUser_Response = _reflection.GeneratedProtocolMessageType('CSiteLicense_LCSAssociateUser_Response', (_message.Message,), {
  'DESCRIPTOR' : _CSITELICENSE_LCSASSOCIATEUSER_RESPONSE,
  '__module__' : 'steammessages_site_license_pb2'
  # @@protoc_insertion_point(class_scope:CSiteLicense_LCSAssociateUser_Response)
  })
_sym_db.RegisterMessage(CSiteLicense_LCSAssociateUser_Response)

CSiteLicense_ClientSeatCheckout_Request = _reflection.GeneratedProtocolMessageType('CSiteLicense_ClientSeatCheckout_Request', (_message.Message,), {
  'DESCRIPTOR' : _CSITELICENSE_CLIENTSEATCHECKOUT_REQUEST,
  '__module__' : 'steammessages_site_license_pb2'
  # @@protoc_insertion_point(class_scope:CSiteLicense_ClientSeatCheckout_Request)
  })
_sym_db.RegisterMessage(CSiteLicense_ClientSeatCheckout_Request)

CSiteLicense_ClientSeatCheckout_Response = _reflection.GeneratedProtocolMessageType('CSiteLicense_ClientSeatCheckout_Response', (_message.Message,), {
  'DESCRIPTOR' : _CSITELICENSE_CLIENTSEATCHECKOUT_RESPONSE,
  '__module__' : 'steammessages_site_license_pb2'
  # @@protoc_insertion_point(class_scope:CSiteLicense_ClientSeatCheckout_Response)
  })
_sym_db.RegisterMessage(CSiteLicense_ClientSeatCheckout_Response)

CSiteLicense_ClientGetAvailableSeats_Request = _reflection.GeneratedProtocolMessageType('CSiteLicense_ClientGetAvailableSeats_Request', (_message.Message,), {
  'DESCRIPTOR' : _CSITELICENSE_CLIENTGETAVAILABLESEATS_REQUEST,
  '__module__' : 'steammessages_site_license_pb2'
  # @@protoc_insertion_point(class_scope:CSiteLicense_ClientGetAvailableSeats_Request)
  })
_sym_db.RegisterMessage(CSiteLicense_ClientGetAvailableSeats_Request)

CSiteLicense_ClientGetAvailableSeats_Response = _reflection.GeneratedProtocolMessageType('CSiteLicense_ClientGetAvailableSeats_Response', (_message.Message,), {
  'DESCRIPTOR' : _CSITELICENSE_CLIENTGETAVAILABLESEATS_RESPONSE,
  '__module__' : 'steammessages_site_license_pb2'
  # @@protoc_insertion_point(class_scope:CSiteLicense_ClientGetAvailableSeats_Response)
  })
_sym_db.RegisterMessage(CSiteLicense_ClientGetAvailableSeats_Response)


DESCRIPTOR._options = None

_SITEMANAGERCLIENT = _descriptor.ServiceDescriptor(
  name='SiteManagerClient',
  full_name='SiteManagerClient',
  file=DESCRIPTOR,
  index=0,
  serialized_options=b'\202\265\030!Local site manager client service\300\265\030\002',
  create_key=_descriptor._internal_create_key,
  serialized_start=1567,
  serialized_end=2114,
  methods=[
  _descriptor.MethodDescriptor(
    name='IncomingClient',
    full_name='SiteManagerClient.IncomingClient',
    index=0,
    containing_service=None,
    input_type=_CSITEMANAGERCLIENT_INCOMINGCLIENT_REQUEST,
    output_type=_CSITEMANAGERCLIENT_INCOMINGCLIENT_RESPONSE,
    serialized_options=b'\202\265\030@Rack informing site manager of new inbound client PSK connection',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ClientSeatCheckoutNotification',
    full_name='SiteManagerClient.ClientSeatCheckoutNotification',
    index=1,
    containing_service=None,
    input_type=_CSITELICENSE_CLIENTSEATCHECKOUT_NOTIFICATION,
    output_type=steammessages__unified__base__pb2._NORESPONSE,
    serialized_options=b'\202\265\0301Notify client directly about seat checkout result',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='TrackedPaymentsNotification',
    full_name='SiteManagerClient.TrackedPaymentsNotification',
    index=2,
    containing_service=None,
    input_type=_CSITEMANAGERCLIENT_TRACKEDPAYMENTS_NOTIFICATION,
    output_type=steammessages__unified__base__pb2._NORESPONSE,
    serialized_options=b'\202\265\0308Rack informing site manager of existing tracked payments',
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_SITEMANAGERCLIENT)

DESCRIPTOR.services_by_name['SiteManagerClient'] = _SITEMANAGERCLIENT


_SITELICENSE = _descriptor.ServiceDescriptor(
  name='SiteLicense',
  full_name='SiteLicense',
  file=DESCRIPTOR,
  index=1,
  serialized_options=b'\202\265\030\024Site License service',
  create_key=_descriptor._internal_create_key,
  serialized_start=2117,
  serialized_end=2970,
  methods=[
  _descriptor.MethodDescriptor(
    name='InitiateAssociation',
    full_name='SiteLicense.InitiateAssociation',
    index=0,
    containing_service=None,
    input_type=_CSITELICENSE_INITIATEASSOCIATION_REQUEST,
    output_type=_CSITELICENSE_INITIATEASSOCIATION_RESPONSE,
    serialized_options=b'\202\265\0302Client is requesting connetion info for local site',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='LCSAuthenticate',
    full_name='SiteLicense.LCSAuthenticate',
    index=1,
    containing_service=None,
    input_type=_CSITELICENSE_LCSAUTHENTICATE_REQUEST,
    output_type=_CSITELICENSE_LCSAUTHENTICATE_RESPONSE,
    serialized_options=b'\202\265\030 Local cafe server authentication',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='LCSAssociateUser',
    full_name='SiteLicense.LCSAssociateUser',
    index=2,
    containing_service=None,
    input_type=_CSITELICENSE_LCSASSOCIATEUSER_REQUEST,
    output_type=_CSITELICENSE_LCSASSOCIATEUSER_RESPONSE,
    serialized_options=b'\202\265\0304Local cafe server wants to grant user access to site',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ClientSeatCheckout',
    full_name='SiteLicense.ClientSeatCheckout',
    index=3,
    containing_service=None,
    input_type=_CSITELICENSE_CLIENTSEATCHECKOUT_REQUEST,
    output_type=_CSITELICENSE_CLIENTSEATCHECKOUT_RESPONSE,
    serialized_options=b'\202\265\030/Client is requesting use of a site license seat',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ClientGetAvailableSeats',
    full_name='SiteLicense.ClientGetAvailableSeats',
    index=4,
    containing_service=None,
    input_type=_CSITELICENSE_CLIENTGETAVAILABLESEATS_REQUEST,
    output_type=_CSITELICENSE_CLIENTGETAVAILABLESEATS_RESPONSE,
    serialized_options=b'\202\265\030<Client is requesting the number of currently available seats',
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_SITELICENSE)

DESCRIPTOR.services_by_name['SiteLicense'] = _SITELICENSE

SiteManagerClient = service_reflection.GeneratedServiceType('SiteManagerClient', (_service.Service,), dict(
  DESCRIPTOR = _SITEMANAGERCLIENT,
  __module__ = 'steammessages_site_license_pb2'
  ))

SiteManagerClient_Stub = service_reflection.GeneratedServiceStubType('SiteManagerClient_Stub', (SiteManagerClient,), dict(
  DESCRIPTOR = _SITEMANAGERCLIENT,
  __module__ = 'steammessages_site_license_pb2'
  ))


SiteLicense = service_reflection.GeneratedServiceType('SiteLicense', (_service.Service,), dict(
  DESCRIPTOR = _SITELICENSE,
  __module__ = 'steammessages_site_license_pb2'
  ))

SiteLicense_Stub = service_reflection.GeneratedServiceStubType('SiteLicense_Stub', (SiteLicense,), dict(
  DESCRIPTOR = _SITELICENSE,
  __module__ = 'steammessages_site_license_pb2'
  ))


# @@protoc_insertion_point(module_scope)
